/*
 * @(#)nfsagent.h	1.1 10/7/91 Interphase
 */

/***************************************************************************
 * This code is copyright 1991 by Interphase Corporation.		   *
 * All rights reserved.							   *
 *									   *
 * This code contains confidential information and trade secrets of	   *
 * Interphase Corporation which shall not be reproduced or	           *
 * transferred to other programs or disclosed to others or used for	   *
 * manufacturing or any other purpose without prior written permission of  *
 * Interphase Corporation.  Use of copyright notice is precautionary       *
 * and does not imply publication or intent thereof.			   *
 ***************************************************************************/

/*
 * from nfsagent.h	1.1 1/21/91 Interphase
 */

#ifndef _NFSAGENT_H
#define _NFSAGENT_H

/*
 * OMNI NFS agent/dispatcher message based interface.
 *
 * NFS service processing is substantially offloaded onto the
 * OMNI Protocol Engine (OPE), an intelligent board capable of
 * executing those compute intensive portions of the NFS/RPC/XDR
 * service protocols and all of the underlying UDP/IP transport.
 *
 * The NFS service request is received by OPE and propagated up to
 * the NFS agent level wherein its contents are XDR decoded directly
 * into the DVMA address space per the data structures defined herein
 * and control passed to the OPE controller.  The OPE controller then
 * interrupts the OPE driver that awakens the NFS dispatcher (omni_nfsd).
 * The NFS dispatcher services the NFS request, generates a reply in the
 * DVMA address space, and passes control to the OPE driver.  The OPE 
 * driver queues the request for the OPE controller which calls the 
 * NFS agent's reply service routine.  The reply service routine
 * XDR encodes and sends the reply directly from the DVMA space.
 *
 * The relevant layers are:
 *
 *		|----------------|
 *		|  vfs interface |
 *		|----------------|
 *		|   nfs server	 |
 *		|----------------|
 *		| nfs dispatcher |
 *		|----------------|
 *		|   ope driver	 |
 *		------------------
 *			|
 *		       DVMA
 *		        |
 *		------------------
 *		| ope controller |
 *		|----------------|
 *		|    nfs agent   |
 *		|----------------|
 *		|     rpc/xdr	 |
 *		|----------------|
 */


/*
 * Agent's service request identification cookie (aid).
 * 
 * Generated by the NFS agent to uniquely denote a service request.
 * Passed by the agent to the dispatcher with the request.  Held 
 * but not examined by the dispatcher, and returned to the agent
 * with the reply.  
 *
 * Internal to the agent, the aid is a ptr to the state record for 
 * an in-process service request.  It is unique per the double:
 *
 *		{client, xid}
 *
 * where, the client's name is maintained in a reference counted
 * list and xid is the NFS service request transaction identifier.
 */
typedef caddr_t	nfsagentid_t;


/*
 * Dispatcher's export information cookie (exid).
 *
 * Generated by the NFS dispatcher to uniquely denote an exported file
 * system information record.  Passed to agent when export record is
 * registered with OPE and returned to dispatcher as *hint* for
 * locating dispatchers corresponding export record.
 *
 * Internal to the dispatcher, the exid is a ptr to an exportinfo
 * structure.  Should the exid no longer be valid, rpc weakauth is
 * replied to agent.
 */
typedef caddr_t	nfsexportid_t;

#define	exidNULL	((caddr_t) 0)	/* export id not registered */

/*
 * RPC OPE msg data format.
 *
 * OPE msg hdrs are typed and point to specific OPE msg data
 * formats: this structure describes the RPC OPE msg data format.
 * Storage is allocated out of the OPE DVMA address space.  The
 * size of storage is determined by the size of this structure,
 * plus the sum of the DVMA lengths for agent, arguments, and
 * results; ie. an NFS agent's RPC OPE msg will appear:
 *
 *		-------------------------
 *		| fix size rpcopedata_t |
 *		-------------------------
 *		| var size arguments    |---->|
 *		-------------------------  or |
 *		| var size results      |---->|
 *		-------------------------     |
 *					      v
 *			    ------------------- 
 *                          |    optional
 *                          v
 *		-------------------------
 *		| var size data block   |
 *		-------------------------
 *
 *  The optional data block can be allocated for either
 *  variable sized argument data or result data.
 */
typedef struct rpcopedata {
	unsigned int rod_board;	      /* requestor controller no. */
	unsigned int rod_bufram;      /* board-resident buffer */
	unsigned int rod_bufram_size; /* 2K or 9K buffer */
	unsigned int rod_dvma;	      /* host-resident buffer */
	unsigned int rod_dvma_size;	  /* size of dvma buffer */
	unsigned int rod_length;	  /* total vme length to transfer */
	unsigned int rod_mrequest;	  /* mbuf's to gather on request */
	unsigned int rod_mreply;      /* mbuf's to scatter on reply */
	unsigned int rod_mlen;	      /* how much to scatter into mbufs */
	int          rod_status;	  /* status of this request */
	unsigned int rod_xprt;	      /* transport handle for this request*/
	unsigned int rod_dispatch;	  /* pointer to rfsdisp table */
	unsigned int rod_prog;	      /* rpc program number */ 
	unsigned int rod_vers;	      /* rpc version number */ 
	unsigned int rod_proc;	      /* rpc procedure number */ 
	unsigned int rod_xid;	      /* rpc transaction id */
	unsigned int rod_dataoff;	  /* offset into data area */
	unsigned int rod_datalen;	  /* data length */
	struct opaque_auth rod_cred;  /* raw credential from wire */
	struct opaque_auth rod_verf;  /* verification */
	struct sockaddr rod_saddr;    /* socket addr of requestor */
	char         rod_cookarea[MAX_AUTH_BYTES]; /* cooked credential */
	/****************************
	* during request :          *
	*    request arguments area *
	*    optional credential    *   pointed by rod_cred.oa_base
	*****************************
	* during reply :            *
	*    reply area             *
	*    optional verf area     *
	****************************/
} rpcopedata_t;

#define OPE_WEAKAUTH	-1
#define OPE_SYSTEM		-2
#define OPE_DROPPED		-3

#ifdef NFSDEBUG
/*
 * Debugging
 */
int nfsdebug;

#endif
#endif !_NFSAGENT_H
